/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.derquinsej.i18n;

import java.util.Locale;
import java.util.Map;
import java.util.Map.Entry;

import com.google.common.base.Preconditions;
import com.google.common.base.Supplier;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;

/**
 * A localized formatted message.
 * @author Andres Rodriguez
 * @param <T> Localized object type.
 */
public final class LocalizedBuilder<T> implements Supplier<Localized<T>> {
	private final T defaultValue;
	private final Map<Locale, T> values = Maps.newHashMap();

	/**
	 * Constructor.
	 * @param defaultValue.
	 */
	public LocalizedBuilder(final T defaultValue) {
		this.defaultValue = defaultValue;
	}

	public LocalizedBuilder<T> put(Locale locale, T value) {
		Preconditions.checkNotNull(locale, "A locale must be provided.");
		Preconditions.checkNotNull(value, "A localized value must be provided.");
		values.put(locale, value);
		return this;
	}

	public LocalizedBuilder<T> putAll(Map<? extends Locale, ? extends T> values) {
		if (values == null || values.isEmpty()) {
			return this; // nothing to do
		}
		// Check preconditions
		for (Entry<? extends Locale, ? extends T> entry : values.entrySet()) {
			Preconditions.checkNotNull(entry.getKey(), "Null locales not allowed.");
			Preconditions.checkNotNull(entry.getValue(), "Null values not allowed.");
		}
		this.values.putAll(values);
		return this;
	}

	public Localized<T> get() {
		return new LocalizedValue<T>(defaultValue, values);
	}

	private static final class LocalizedValue<T> extends AbstractLocalized<T> {
		private final T defaultValue;
		private final ImmutableMap<Locale, T> values;

		/**
		 * Default constructor.
		 * @param pattern Pattern.
		 * @param args Arguments.
		 */
		private LocalizedValue(T defaultValue, Map<Locale, T> values) {
			this.defaultValue = defaultValue;
			this.values = ImmutableMap.copyOf(values);
		}

		/*
		 * (non-Javadoc)
		 * @see net.sf.derquinsej.i18n.AbstractLocalized#get()
		 */
		@Override
		public T get() {
			return defaultValue;
		}

		/*
		 * (non-Javadoc)
		 * @see net.sf.derquinsej.i18n.Localized#get(java.util.Locale)
		 */
		public T get(Locale locale) {
			while (locale != null) {
				T value = values.get(locale);
				if (value != null) {
					return value;
				}
				locale = Locales.getParent(locale);
			}
			return defaultValue;
		}
	}

}
